/* OpenProcessing Tweak of *@*http://www.openprocessing.org/sketch/100498*@* */
/* !do not delete the line above, required for linking your tweak if you upload again */
PImage levels;

MapMesh mesh;

MapParticule[] parts;
int pnum = 1000;

int sampling = 12;
float elevation = 120;

boolean showimage = false;
boolean drawedges = false;
boolean drawfaces = false;

void setup() {

  size( 1024, 768, P3D );
  levels = loadImage( "picardie.png" );
  
  mesh = new MapMesh();
  mesh.process( levels, elevation, sampling );

  parts = new MapParticule[ pnum ];
  for ( int i=0; i < pnum; i++ )
    parts[i] = new MapParticule( 
      random( 0.1f * mesh.width, 0.9f * mesh.width ), 
      random( 0.1f * mesh.height, 0.9f * mesh.height ),
      elevation );
      
  background( 0 );

}

void draw() {

  for ( int i=0; i < parts.length; i++ )
    parts[i].update( mesh );

  background( 0 );
  translate( width * 0.5f, height * 0.5f, -300 );
  rotateX( -mouseY * 0.03f );
  rotateZ( mouseX * 0.03f );
  translate( -levels.width * 0.5f, -levels.height * 0.5f, 0  );
  if ( showimage )
    image( levels, 0, 0 );
  noFill();
  
  stroke( 0, 255, 249 );
  for ( int i=0; i < parts.length; i++ ) {
    strokeWeight( 2 );
    point( parts[i].positions[ 0 ], parts[i].positions[ 1 ], parts[i].positions[ 2 ] );
    /*
    // rendering slope
    pushMatrix();
    translate( parts[i].positions[ 0 ], parts[i].positions[ 1 ], parts[i].positions[ 2 ] );
    rotateZ( -mouseX * 0.03f );
    rotateX( mouseY * 0.03f );
    noStroke();
    fill( 255 );
    if ( parts[i].positions[ 3 ] < -0.001f ) {
      translate( 0,-10,0 );
      beginShape();
      vertex( 0,-7,0 );
      vertex( -6,7,0 );
      vertex( 6,7,0 );
      endShape( CLOSE );
    } else if ( parts[i].positions[ 3 ] > 0.001f ) {
      translate( 0,10,0 );
      beginShape();
      vertex( 0,7,0 );
      vertex( -6,-7,0 );
      vertex( 6,-7,0 );
      endShape( CLOSE );
    }
    popMatrix();
    */
  }
  
  if ( drawedges && !drawfaces ) {
    strokeWeight(  1 );
    stroke( 127 );
    for ( int r = 0; r < mesh.grid[ 1 ]; r++ ) {
    for ( int c = 0; c < mesh.grid[ 0 ]; c++ ) {
      int p = c + r * mesh.grid[ 0 ];
      if ( c < mesh.grid[ 0 ] - 1 ) {
        line( 
          mesh.points[ p ].position.x,
          mesh.points[ p ].position.y,
          mesh.points[ p ].position.z,
          mesh.points[ p + 1 ].position.x,
          mesh.points[ p + 1 ].position.y,
          mesh.points[ p + 1 ].position.z
        );
      }
      if ( r < mesh.grid[ 1 ] - 1 ) {
        line( 
          mesh.points[ p ].position.x,
          mesh.points[ p ].position.y,
          mesh.points[ p ].position.z,
          mesh.points[ p + mesh.grid[ 0 ] ].position.x,
          mesh.points[ p + mesh.grid[ 0 ] ].position.y,
          mesh.points[ p + mesh.grid[ 0 ] ].position.z
        );
      }
    }
    }
  }
  if ( drawfaces ) {
    strokeWeight( 1 );
    for ( int f = 0; f < mesh.faces.length; f++ ) {
      Mappoint[] pts = mesh.faces[ f ].pts;
      noStroke();
      beginShape();
      fill( mesh.centers[ f ].position.z / elevation * 255.f );
      for ( int p = 0; p < pts.length; p++ )
        vertex( pts[ p ].position.x, pts[ p ].position.y, pts[ p ].position.z );
      endShape( CLOSE );
      stroke( 255,0,0 );
      pushMatrix();
      translate( mesh.centers[ f ].position.x, mesh.centers[ f ].position.y, mesh.centers[ f ].position.z );
      line( 0,0,0, mesh.faces[ f ].normale.x * 20, mesh.faces[ f ].normale.y * 20, mesh.faces[ f ].normale.z * 20 );
      popMatrix();
    }
  }
  
  // bundaries
  stroke( 255 );
  strokeWeight( 2 );
  noFill();
  pushMatrix();
  rect( 0, 0, levels.width, levels.height );
  translate( 0, 0, elevation );
  stroke( 80 );
  rect( 0, 0, levels.width, levels.height );
  popMatrix();

}

void keyPressed( ) {
  
  if ( key == 'e' ) {
    drawedges = !drawedges;
  } else if ( key == 'f' ) {
    drawfaces = !drawfaces;
  } else if ( key == 'i' ) {
    showimage = !showimage;
  } else if ( key == 'r' ) {
    for ( int i=0; i < pnum; i++ )
      parts[i].reset( random( 0.1f * mesh.width, 0.9f * mesh.width ), random( 0.1f * mesh.height, 0.9f * mesh.height ), elevation );
  } else {
    println( keyCode );
  }

}

